@@ -37,6 +37,7 @@ public:
 	virtual DATA get_connection_weight(int connection) = 0;
 	virtual bool set_connection_weight(int connection, DATA value) = 0;
 	virtual bool set_misc(DATA * data, int dimension) = 0;
+	virtual bool get_misc(DATA * buffer, int dimension) = 0;							// added for nnlib2Rcpp 0.1.11
 	virtual	bool setup (layer PTR source_layer, layer PTR destin_layer, bool PTR error_flag_to_use, bool fully_connect_layers = false) = 0;
 	virtual	bool setup (string name, layer PTR source_layer, layer PTR destin_layer, bool PTR error_flag_to_use, bool fully_connect_layers = false) = 0;
     virtual	bool setup (string name, layer PTR source_layer, layer PTR destin_layer, bool PTR error_flag_to_use, bool fully_connect_layers, DATA min_random_weight, DATA max_random_weight) = 0;
@@ -83,12 +84,13 @@ class Connection_Set : public connection_set
  DATA get_connection_weight(int connection);					// returns 0 if not successful
  bool set_connection_weight(int connection, DATA value);
 
- // Note: the following set_connection_weight... functions are added for initializing weights. Using in processing is not recommented.
+ // Note: the following set_connection_weight... functions are added for initializing weights. Using in processing is not recommended.
  void set_connection_weights (DATA value);
  void set_connection_weights_random(DATA min_random_value, DATA max_random_value);
  bool set_connection_weight (const int source_pe, const int destin_pe, const DATA new_weight);
 
- // set values in misc internal register variable (found in connections)
+ // get or set values in misc internal register variable (found in connections)
+ bool get_misc(DATA * buffer, int dimension);
  bool set_misc(DATA * data, int dimension);
 
  string description ();
@@ -109,8 +111,8 @@ class Connection_Set : public connection_set
  bool has_source_layer();
  bool has_destin_layer();
 
- void encode();													// (virtual in component) may be overiden by derived classes with specific layer functiobality.
- void recall();													// (virtual in component) may be overiden by derived classes with specific layer functiobality.
+ void encode();													// (virtual in component) may be overridden by derived classes with specific layer functiobality.
+ void recall();													// (virtual in component) may be overridden by derived classes with specific layer functiobality.
 
  bool add_connection(const int source_pe, const int destin_pe, const DATA initial_weight);
  bool remove_connection(int connection_number);
@@ -502,7 +504,7 @@ void Connection_Set<CONNECTION_TYPE>::to_stream (std::ostream REF s)
 }
 
 //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-// may be overiden by derived classes.
+// may be overridden by derived classes.
 
 template <class CONNECTION_TYPE>
 void Connection_Set<CONNECTION_TYPE>::encode()
@@ -595,6 +597,26 @@ bool Connection_Set<CONNECTION_TYPE>::has_destin_layer()
 return (mp_destin_layer != NULL);
 }
 
+//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+// get misc variable in connections
+
+template <class CONNECTION_TYPE>
+bool Connection_Set<CONNECTION_TYPE>::get_misc(DATA * buffer, int dimension)
+{
+	if (NOT no_error()) return false;
+	if (buffer == NULL) return false;
+	if (dimension NEQL size())
+	{ warning ("Incompatible vector dimension (number of connections vs vector length)");
+		return false; }
+	if(NOT connections.goto_first()) return false;
+	for (int i = 0; i < dimension; i++)
+		{
+		buffer[i] = connections.current().misc;                    // get data from respective connection misc
+		if(NOT connections.goto_next()) i=dimension;
+		}
+	return true;
+}
+
 //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 // set misc variable in connections
 
@@ -608,10 +630,10 @@ bool Connection_Set<CONNECTION_TYPE>::set_misc(DATA * data, int dimension)
 		return false; }
 	if(NOT connections.goto_first()) return false;
 	for (int i = 0; i < dimension; i++)
-		{
+	{
 		connections.current().misc = data[i];                    // sets data to respective connection misc
 		if(NOT connections.goto_next()) i=dimension;
-		}
+	}
 	return true;
 }
 
