@@ -37,12 +37,14 @@ public:
 	virtual string item_description(int item) = 0;
 	virtual void from_stream(std::istream REF s) = 0;                                   // read layer from stream
 	virtual void to_stream(std::ostream REF s) = 0;                                     // write layer to stream
-	virtual bool set_misc(DATA * data, int dimension) = 0;
 	virtual bool set_output(DATA * data, int dimension) = 0;
 	virtual bool set_biases(DATA * data, int dimension) = 0;							// added for nnlib2Rcpp 0.1.10
 	virtual bool set_bias_at(int index, DATA d) = 0;									// added for nnlib2Rcpp 0.1.10
 	virtual bool get_biases(DATA * buffer, int dimension) = 0;							// added for nnlib2Rcpp 0.1.10
+	virtual bool get_misc(DATA * buffer, int dimension) = 0;							// added for nnlib2Rcpp 0.1.11
+	virtual bool set_misc(DATA * data, int dimension) = 0;
 	virtual DATA get_bias_from(int index) = 0;											// added for nnlib2Rcpp 0.1.10
+	virtual bool get_input(DATA * buffer, int dimension) = 0;						    // added for nnlib2Rcpp 0.2.0
 };
 
 /*-----------------------------------------------------------------------*/
@@ -75,21 +77,24 @@ public:
 	void from_stream (std::istream REF s);                                 // read layer from stream
 	void to_stream (std::ostream REF s);                                   // write layer to stream
 
-	bool input_data_from_vector(DATA * data, int dimension);               // overides virtual method in data_receiver, sets values to pe inputs
-	bool output_data_to_vector(DATA * buffer, int dimension);              // overides virtual method in data_provider, gets values from pe outputs
-	bool send_input_to(int index, DATA d);                                 // overides virtual method in data_receiver, sets value to corresponding pe input sets this input to respective pe input (and also appends to pe's list of input values)
-	DATA get_output_from (int index);                                      // overides virtual method in data_provider, gets value from corresponding pe output
+	bool input_data_from_vector(DATA * data, int dimension);               // overrides virtual method in data_receiver, sets values to pe inputs
+	bool output_data_to_vector(DATA * buffer, int dimension);              // overrides virtual method in data_provider, gets values from pe outputs
+	bool send_input_to(int index, DATA d);                                 // overrides virtual method in data_receiver, sets value to corresponding pe input sets this input to respective pe input (and also appends to pe's list of input values)
+	DATA get_output_from (int index);                                      // overrides virtual method in data_provider, gets value from corresponding pe output
 
+	bool get_misc(DATA * buffer, int dimension);						   // added for nnlib2Rcpp 0.1.11
 	bool set_misc(DATA * data, int dimension);							   // set values in misc internal register variables in layer pes
+
 	bool set_output(DATA * data, int dimension);						   // overwrites current output registers in layer pes, with the provided data values
 
 	bool set_biases(DATA * data, int dimension);						   // overwrites current bias registers in layer pes, with the provided data values
 	bool set_bias_at(int index, DATA d);								   // overwrites current bias register in pe, with the provided data values
 	bool get_biases(DATA * buffer, int dimension);						   // added for nnlib2Rcpp 0.1.10
 	DATA get_bias_from(int index);										   // added for nnlib2Rcpp 0.1.10
+	bool get_input(DATA * buffer, int dimension);						   // added for nnlib2Rcpp 0.2.0
 
-	void encode();                                                         // (virtual in component) may be overiden by derived classes with specific layer functiobality.
-	void recall();                                                         // (virtual in component) may be overiden by derived classes with specific layer functiobality.
+	void encode();                                                         // (virtual in component) may be overridden by derived classes with specific layer functiobality.
+	void recall();                                                         // (virtual in component) may be overridden by derived classes with specific layer functiobality.
 
 	//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 
@@ -234,7 +239,7 @@ void Layer<PE_TYPE>::to_stream(std::ostream REF s)
 }
 
 //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-// overides virtual method in data_receiver, sets values to pe inputs
+// overrides virtual method in data_receiver, sets values to pe inputs
 // (sets this input to respective pe input and to received_values,
 // clearing all previous ones)
 
@@ -256,7 +261,7 @@ bool Layer<PE_TYPE>::input_data_from_vector(DATA* data, int dimension)
 }
 
 //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-// overides virtual method in data_provider, gets values from pe outputs
+// overrides virtual method in data_provider, gets values from pe outputs
 // (gets the output value from respective pes and copies it to buffer)
 
 template <class PE_TYPE>
@@ -270,7 +275,7 @@ bool Layer<PE_TYPE>::output_data_to_vector(DATA* buffer, int dimension)
 }
 
 //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-// overides virtual method in data_receiver, sets value to corresponding pe input
+// overrides virtual method in data_receiver, sets value to corresponding pe input
 
 template <class PE_TYPE>
 bool Layer<PE_TYPE>::send_input_to(int index, DATA d)
@@ -283,7 +288,7 @@ bool Layer<PE_TYPE>::send_input_to(int index, DATA d)
 }
 
 //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-// overides virtual method in data_provider, gets value from corresponding pe output
+// overrides virtual method in data_provider, gets value from corresponding pe output
 
 template <class PE_TYPE>
 DATA Layer<PE_TYPE>::get_output_from(int index)
@@ -294,6 +299,22 @@ DATA Layer<PE_TYPE>::get_output_from(int index)
 	return(pes[index].output);
 }
 
+//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+// get misc variable values from pes
+
+template <class PE_TYPE>
+bool Layer<PE_TYPE>::get_misc(DATA * buffer, int dimension)
+{
+	if (NOT no_error()) return false;
+	if (buffer == NULL) return false;
+	if (dimension NEQL size())
+	{ warning ("Incompatible vector dimension (number of PEs vs vector length)");
+		return false; }
+	for (int i = 0; i < dimension; i++)
+		buffer[i] = pes[i].misc;                    // gets respective pe misc
+	return true;
+}
+
 //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 // set misc variable in pes
 
@@ -373,6 +394,23 @@ bool Layer<PE_TYPE>::get_biases(DATA * buffer, int dimension)
 	return true;
 }
 
+
+
+//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+// get (estimate of) input values
+//
+// Important!!! see implementation of preview_current_input as this is an ESTIMATE and may be affected by PE specific implementation!
+
+template <class PE_TYPE>
+bool Layer<PE_TYPE>::get_input(DATA * buffer, int dimension)
+	{
+		if (NOT no_error()) return false;
+		if (buffer == NULL) return false;
+		if (dimension NEQL size()) { warning ("Incompatible output vector dimension (number of PEs vs vector length)"); return false; }
+		for (int i = 0; i < dimension; i++) buffer[i] = pes[i].preview_current_input();
+		return true;
+	}
+
 //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 // get bias values
 // added for nnlib2Rcpp 0.1.10 (see Github issue https://github.com/VNNikolaidis/nnlib2Rcpp/issues/13)
@@ -401,7 +439,7 @@ bool Layer<PE_TYPE>::move_all_pe_input_to_output()
 }
 
 //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-// should be overiden by derived classes.
+// should be overridden by derived classes.
 
 template <class PE_TYPE>
 void Layer<PE_TYPE>::encode()
@@ -423,7 +461,9 @@ void Layer<PE_TYPE>::recall()
 //-------------------------------------------------------------------------
 // For explicit instantiation of layer template per pe type (not needed
 // as implementation is in this header) use code similar to:
-// template class layer<pe>;                        // instantiate a layer of generic pes
+
+
+// template class layer<pe>;   // (instantiate a layer of generic pes)
 
 //-------------------------------------------------------------------------
 // layer of generic "dumb" pes where most processing will be done in layer code:
