@@ -82,6 +82,11 @@ void nn::reset(bool clear_additional_parameters)
  m_topology_component_for_output = -1;
  }
 
+
+//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+
+void nn::reset() {reset(true);}
+
 //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 
 DATA nn::encode_u(DATA PTR input, int input_dim, int iteration)
@@ -94,7 +99,7 @@ DATA nn::encode_u(DATA PTR input, int input_dim, int iteration)
 // performs a typical supervised encoding, assuming input will go to first layer (unless otherwise defined), while
 // desired output will be presented as _input_ to the last layer in topology (unless otherwise defined).
 // May return indication of encoding success.
-// Should be overiden with needed behaviour, as this is very basic.
+// Should be overridden with needed behaviour, as this is very basic.
 
 DATA nn::encode_s(DATA PTR input, int input_dim, DATA PTR desired_output, int output_dim, int iteration)
  {
@@ -128,7 +133,7 @@ DATA nn::encode_s(DATA PTR input, int input_dim, DATA PTR desired_output, int ou
 
 //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 // Performs a typical recall, in the direction from input to output
-// in topology. Assumes data is already there. Overide if needed.
+// in topology. Assumes data is already there. Override if needed.
 
 void nn::recall()
 {
@@ -150,7 +155,7 @@ else
 
 //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 // Performs a typical feed forward encode, in the direction from input to output
-// in topology. Assumes data is already there. Overide if needed.
+// in topology. Assumes data is already there. Override if needed.
 
 void nn::encode()
 {
@@ -171,38 +176,48 @@ else
 }
 
 //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-// overide if necessary...
+// override if necessary...
 
 int nn::input_dimension()
  {
- // we are probably making too many unneeded checks here
+ // we are probably making too many unneeded checks here:
  if(m_topology_component_for_input<0) return 0;
  if(topology.is_empty()) return 0;
  if(topology.size()<=m_topology_component_for_input)return 0;
- if(topology[m_topology_component_for_input]->type()!=cmpnt_layer) return 0;
- layer PTR p_input_layer = reinterpret_cast<layer PTR>(topology[m_topology_component_for_input]);
- if (p_input_layer!=NULL) return p_input_layer->size();
+ if(!component_accepts_input(m_topology_component_for_input)) return 0;
+ return topology[m_topology_component_for_input]->size();
+ // remaining code is old version's, not used:
+ if(topology[m_topology_component_for_input]->type()==cmpnt_layer)
+ {
+ 	layer PTR p_input_layer = reinterpret_cast<layer PTR>(topology[m_topology_component_for_input]);
+ 	if (p_input_layer!=NULL) return p_input_layer->size();
+ }
  return 0;
  }
 
 //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-// overide if necessary...
+// override if necessary...
 
 int nn::output_dimension()
  {
- // we are probably making too many unneeded checks here
+ // we are probably making too many unneeded checks here:
  if(m_topology_component_for_output<0) return 0;
  if(topology.is_empty()) return 0;
  if(topology.size()<=m_topology_component_for_output)return 0;
- if(topology[m_topology_component_for_output]->type()!=cmpnt_layer) return 0;
- layer PTR p_output_layer = reinterpret_cast<layer PTR>(topology[m_topology_component_for_output]);
- if (p_output_layer!=NULL) return p_output_layer->size();
+ if(!component_provides_output(m_topology_component_for_output)) return 0;
+ return topology[m_topology_component_for_output]->size();
+ // remaining code is old version's, not used:
+ if(topology[m_topology_component_for_output]->type()==cmpnt_layer)
+ {
+ 	layer PTR p_output_layer = reinterpret_cast<layer PTR>(topology[m_topology_component_for_output]);
+ 	if (p_output_layer!=NULL) return p_output_layer->size();
+ }
  return 0;
  }
 
 //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 // (required by data_receiver class)
-// attemps to place data on the first component, unless otherwise defined.
+// attempts to place data on the first component, unless otherwise defined.
 // assuming it [is a layer that] can input data (data_receiver)
 
 bool nn::input_data_from_vector(DATA * data, int dimension)
@@ -224,7 +239,7 @@ bool nn::input_data_from_vector(DATA * data, int dimension)
 // attemps to place data on the m_topology_component_for_input component
 // (sets it to first if not specified)
 // assuming it [is a layer that] can input data (data_receiver)
-// overides virtual method in data_receiver, as above, sets value to corresponding pe input
+// overrides virtual method in data_receiver, as above, sets value to corresponding pe input
 
 bool nn::send_input_to(int index, DATA d)
 {
@@ -258,10 +273,10 @@ bool nn::output_data_to_vector(DATA * buffer, int dimension)
 
 //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 // (required by data_provider class)
-// attemps to get data from the m_topology_component_for_output component in topology
+// attempts to get data from the m_topology_component_for_output component in topology
 // (sets it to last if not specified)
 // assuming it [is a layer that] outputs data (data_provider)
-// overides virtual method in data_provider, as above, gets value from corresponding pe output
+// overrides virtual method in data_provider, as above, gets value from corresponding pe output
 
 DATA nn::get_output_from (int index)
 {
@@ -284,8 +299,8 @@ bool nn::set_component_for_input(int index)
  if((index<0) OR
     (index>=topology.size()))
      {warning("Requested input component is not in topology"); return false;}
- if(topology[index]->type()!=cmpnt_layer)
-     {warning("Requested input component is not a layer"); return false;}
+ if(!component_accepts_input(index))
+     {warning("Requested component does not support direct input"); return false;}
  m_topology_component_for_input = index;
  #ifndef NNLIB2_WITH_GUI
  // TEXTOUT << "Note: Using NN component in topology index position " << m_topology_component_for_input << " for input.\n";
@@ -301,8 +316,8 @@ bool nn::set_component_for_output(int index)
  if((index<0) OR
    (index>=topology.size()))
     {warning("Requested output component is not in topology"); return false;}
- if(topology[index]->type()!=cmpnt_layer)
-    {warning("Requested output component must be a layer"); return false;}
+ if(!component_provides_output(index))
+    {warning("Requested component type does not maintain, provide or report output"); return false;}
  m_topology_component_for_output = index;
  #ifndef NNLIB2_WITH_GUI
  // TEXTOUT << "Note: Using NN component in topology index position " << m_topology_component_for_output << " for output.\n";
@@ -433,11 +448,11 @@ string nn::outline (bool show_first_index_as_one)
     {
     if(show_first_index_as_one)
      {
-     s << "@" << c+1 << " (c=" << c << ")";
+     s << "@ " << c+1; // << " (c=" << c << ")";
      }
     else
      {
-     s << "@" << c;
+     s << "@ " << c;
      }
     s << " component (id=" << topology.current()->id() << ")";
     s << " is " << topology.current()->description();
@@ -478,6 +493,17 @@ bool nn::add_connection_set(connection_set * p_connection_set)
   return false;
 }
 
+bool nn::add_aux_control(aux_control * p_aux_control)
+{
+	if(add_component(p_aux_control))
+	{
+		p_aux_control->set_error_flag(my_error_flag());
+		return true;
+	}
+	return false;
+}
+
+
 //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 // Trigger encode for component at specified topology index position
 
@@ -551,13 +577,13 @@ return false;
 
 component * nn::component_from_id(int id)
 {
-if(topology.goto_first())
-  {
-    do
-      if(topology.current()->id()==id) return topology.current();
-    while (topology.goto_next());
-  }
-return NULL;
+	for(int i=0;i<topology.size();i++)
+	{
+		component * cp = topology[i];
+		if(no_error())
+			if(cp->id()==id) return cp;
+	}
+	return NULL;
 }
 
 //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -565,28 +591,49 @@ return NULL;
 
 component * nn::component_from_topology_index(int index)
 {
-  int i=0;
-  if(topology.goto_first())
-  {
-    do
-    {
-      if(index==i) return topology.current();
-      i++;
-    }
-    while (topology.goto_next());
-  }
+  component * cp = topology[index];
+  if(no_error()) return cp;
   return NULL;
 }
 
+//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+// returns true if data_receiver (layer/aux_control)
+
+bool nn::component_accepts_input(int index)
+{
+	component * cp = component_from_topology_index(index);
+	if(cp==NULL) return false;
+	if(cp->type()==cmpnt_layer) return true;
+	if(cp->type()==cmpnt_aux_control) return true;
+	if(dynamic_cast<data_receiver *>(cp)!=NULL) return true;
+	return false;
+}
+
+//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+// returns true if data_provider (layer/aux_control)
+
+bool nn::component_provides_output(int index)
+{
+	component * cp = component_from_topology_index(index);
+	if(cp==NULL) return false;
+	if(cp->type()==cmpnt_layer) return true;
+	if(cp->type()==cmpnt_aux_control) return true;
+	if(dynamic_cast<data_provider *>(cp)!=NULL) return true;
+	return false;
+}
 
 //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 // returns the index position in topology for component with given id, negative (-1) if not available
 
 int nn::component_topology_index_from_id(int id)
 {
-  component PTR pc = component_from_id(id);
-  if(pc==NULL) return -1;
-  return pc->id();
+	for(int i=0;i<topology.size();i++)
+	{
+		component * cp = topology[i];
+		if(no_error())
+			if(cp->id()==id) return i;
+	}
+	return -1;
 }
 
 //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -712,7 +759,7 @@ for(int i=0;i<layer_indexes_in_topology.size()-1;i++)
  all_connected = all_connected AND pair_connected;
  }
 
-if (NOT all_connected)  warning("Could not connect all layer pairs.");
+if (NOT all_connected)  warning("Could not connect all layer pairs (not all layer pairs are linked by connections).");
 bool check_ok = all_connected AND no_error();
 if (set_ready_to_encode_fwd AND check_ok)
  {
@@ -752,6 +799,7 @@ bool nn::connect_layers_at_topology_indexes(                                // c
                                       DATA max_random_weight)               // these two values.
  {
   if(p_connection_set==NULL) return false;
+
   if((source_layer_index<0) OR (source_layer_index>=topology.size())) return false;
   if((destination_layer_index<0) OR (destination_layer_index>=topology.size())) return false;
 
@@ -763,7 +811,17 @@ bool nn::connect_layers_at_topology_indexes(                                // c
   if(p_c1->type()!=cmpnt_layer) { warning("Source is not a layer"); return false;}
   if(p_c2->type()!=cmpnt_layer) { warning("Destination is not a layer"); return false;}
 
-  if(NOT topology.insert(source_layer_index+1,p_connection_set)) return false;
+  if(source_layer_index<destination_layer_index)
+   {if(NOT topology.insert(source_layer_index+1,p_connection_set)) return false;}
+
+  if(source_layer_index==destination_layer_index)
+  {
+  	warning("Source layer equals destination layer, placing connection set below layer in topology");
+  	if(NOT topology.insert(source_layer_index+1,p_connection_set)) return false;
+  }
+
+  if(source_layer_index>destination_layer_index)
+   {if(NOT topology.insert(destination_layer_index+1,p_connection_set)) return false;}
 
   layer PTR p_l1 = reinterpret_cast<layer PTR>(p_c1);
   layer PTR p_l2 = reinterpret_cast<layer PTR>(p_c2);
@@ -814,6 +872,25 @@ connection_set PTR nn::get_connection_set_at(int index)
   return p_cs;
 }
 
+//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+// patch. NULL if not found or not aux_control
+
+aux_control PTR nn::get_aux_control_at(int index)
+{
+	if(index<0) return NULL;
+	if(index>=topology.size()) return NULL;
+	if(topology.is_empty()) return NULL;
+
+	component PTR p_comp = topology[index];
+	if(p_comp==NULL) return NULL;
+
+	if(p_comp->type()!=cmpnt_aux_control) return NULL;
+	// component found and seems to be a aux_control
+
+	aux_control PTR p_ax = reinterpret_cast<aux_control PTR>(p_comp);
+	return p_ax;
+}
+
 //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 // patch.
 
@@ -838,8 +915,9 @@ bool nn::remove_connection(int index, int connection_number)
 
 //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 // patch.
+// Important: see implementation issues.
 
-bool nn::get_input_data_at_component (int index, DATA * buffer, int dimension)
+bool nn::get_input_at_component (int index, DATA * buffer, int dimension)
 {
   if(buffer==NULL) return false;
   if(dimension<=0) return false;
@@ -853,15 +931,14 @@ bool nn::get_input_data_at_component (int index, DATA * buffer, int dimension)
   int num_items = p_comp->size();
   if(num_items!=dimension)
     {
-    warning("Cannot retreive inputs, sizes do not match");
+  	warning("Cannot retreive inputs, sizes do not match");
     return false;
     }
 
   if(p_comp->type()==cmpnt_layer)                // component found and it is a layer
   {
-  layer PTR p_la = reinterpret_cast<layer PTR>(p_comp);
-  for(int i=0;i<num_items;i++) buffer[i] = p_la->PE(i).input;
-  return true;
+  	layer PTR p_la = reinterpret_cast<layer PTR>(p_comp);
+  	return p_la->get_input(buffer, dimension);
   }
 
   if(p_comp->type()==cmpnt_connection_set)       // component found and it is a connection_set
@@ -877,6 +954,22 @@ bool nn::get_input_data_at_component (int index, DATA * buffer, int dimension)
 //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 // patch.
 
+bool nn::set_input_at_component (int index, DATA * data, int dimension)
+{
+	if(data==NULL) return false;
+	if(dimension<=0) return false;
+	if(index<0) return false;
+	if(index>=topology.size()) return false;
+	if(topology.is_empty()) return false;
+	if(!component_accepts_input(index)) return false;
+	data_receiver * pc = dynamic_cast <data_receiver *> (topology[index]);
+	if (pc==NULL) {error(NN_INTEGR_ERR,"Requested component cannot accept data");return false;}
+	return(pc->input_data_from_vector(data,dimension));
+}
+
+//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+// patch.
+
 bool nn::get_weights_at_component (int index, DATA * buffer, int dimension)
 {
   if(buffer==NULL) return false;
@@ -905,6 +998,24 @@ bool nn::get_weights_at_component (int index, DATA * buffer, int dimension)
   return true;
  }
 
+//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+// patch.
+
+bool nn::set_weights_at_component(int index, DATA * data, int dimension)
+{
+	if(data==NULL) return false;
+	if(dimension<=0) return false;
+
+	connection_set PTR p_cs = get_connection_set_at(index);
+	if(p_cs==NULL) {warning("Invalid connection set"); return false;}
+	if(p_cs->size()!=dimension) return false;
+
+	for(int i=0;i<dimension;i++)
+		if(!p_cs->set_connection_weight(i, data[i])) return false;
+
+	return true;
+}
+
 //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 // patch. Returns 0 if not successful
 
@@ -928,6 +1039,18 @@ bool nn::set_weight_at_component(int index, int connection_number, DATA weight)
 //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 // patch
 
+bool nn::get_misc_at_component(int index, DATA * buffer, int dimension)
+{
+	layer PTR p_lay = get_layer_at(index);
+	if (p_lay != NULL) return p_lay->get_misc(buffer,dimension);
+	connection_set PTR p_cs = get_connection_set_at(index);
+	if (p_cs != NULL) return p_cs->get_misc(buffer,dimension);
+	return false;
+}
+
+//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+// patch
+
 bool nn::set_misc_at_component(int index, DATA * data, int dimension)
 {
   layer PTR p_lay = get_layer_at(index);
@@ -937,6 +1060,22 @@ bool nn::set_misc_at_component(int index, DATA * data, int dimension)
   return false;
 }
 
+//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+// patch
+
+bool nn::get_output_from_component (int index, DATA * buffer, int dimension)
+{
+	if(buffer==NULL) return false;
+	if(dimension<=0) return false;
+	if(index<0) return false;
+	if(index>=topology.size()) return false;
+	if(topology.is_empty()) return false;
+	if(!component_accepts_input(index)) return false;
+	data_provider * pc = dynamic_cast <data_provider *> (topology[index]);
+	if (pc==NULL) {error(NN_INTEGR_ERR,"Requested component cannot provide data");return false;}
+	return(pc->output_data_to_vector(buffer,dimension));
+}
+
 //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 // patch, currently only works only for layers
 // overwrites current pe output registers with the provided data values
@@ -955,7 +1094,7 @@ bool nn::set_output_at_component(int index, DATA * data, int dimension)
 bool nn::get_biases_at_component (int index, DATA * buffer, int dimension)
 {
 	layer PTR p_lay = get_layer_at(index);
-	if(p_lay==NULL) {warning("Invalid layer"); return false;}
+	if(p_lay==NULL) {warning("Component is not a layer or is invalid"); return false;}
 	return p_lay->get_biases(buffer,dimension);
 }
 
@@ -965,7 +1104,7 @@ bool nn::get_biases_at_component (int index, DATA * buffer, int dimension)
 DATA nn::get_bias_at_component(int index, int pe)
 {
 	layer PTR p_lay = get_layer_at(index);
-	if(p_lay==NULL) {warning("Invalid layer"); return false;}
+	if(p_lay==NULL) {warning("Component is not a layer or is invalid"); return false;}
 	return p_lay->get_bias_from(pe);
 }
 
