@@ -16,6 +16,7 @@
 
 #include "layer.h"
 #include "connection_set.h"
+#include "aux_control.h"
 
 #ifdef NNLIB2_FOR_MFC_UI
 #include "..\nnlib2.mfcgui\nnlib2_mfc_ui.h"
@@ -50,11 +51,12 @@ class nn : public component, public data_receiver, public data_provider, public
  nn(string name);
  ~nn();
 
- virtual void reset(bool clear_additional_parameters = true);
+ virtual void reset();
+ virtual void reset(bool clear_additional_parameters);
  virtual bool set_additional_parameters(double param1,...);
 
- void encode();                                                         // (required by component class) performs a typical encode, in the direction from input to output. Assumes data is available. Overide if something else is needed.
- void recall();                                                         // (required by component class) Performs a typical recall, in the direction from input to output. Assumes data is available. Overide if something else is needed.
+ void encode();                                                         // (required by component class) performs a typical encode, in the direction from input to output. Assumes data is available. Override if something else is needed.
+ void recall();                                                         // (required by component class) Performs a typical recall, in the direction from input to output. Assumes data is available. Override if something else is needed.
 
  // unsupervised and supervised variations to customize if needed by model.
  // By default, they present the data vectors and use encode() and recall() on the NN topology.
@@ -65,13 +67,13 @@ class nn : public component, public data_receiver, public data_provider, public
  virtual bool recall(DATA PTR input, int dim);
  virtual bool recall(DATA PTR input,int input_dim, DATA PTR output_buffer, int output_dim);
 
- bool input_data_from_vector(DATA * data, int dimension);               // (data_receiver virtual method) attemps to place data on the m_topology_component_for_input component (sets it to first if unspecified), assuming it [is a layer that] can input data (data_receiver)
+ bool input_data_from_vector(DATA * data, int dimension);               // (data_receiver virtual method) attempts to place data on the m_topology_component_for_input component (sets it to first if unspecified), assuming it [is a layer that] can input data (data_receiver)
  bool send_input_to(int index, DATA d);                                 // (data_receiver virtual method) as above, sets value to corresponding pe input
  bool output_data_to_vector(DATA * buffer, int dimension);              // (data_provider virtual method) attemps to get data from the m_topology_component_for_output component (sets it to last if unspecified) component, assuming it [is a layer that] can output data (data_provider)
  DATA get_output_from (int index);                                      // (data_provider virtual method) as above, gets value from corresponding pe output
 
- virtual int input_dimension();                                         // overide if not using topology.
- virtual int output_dimension();                                        // overide if not using topology.
+ virtual int input_dimension();                                         // override if not using topology.
+ virtual int output_dimension();                                        // override if not using topology.
 
  int input_length()     { return input_dimension();  }
  int output_length()    { return output_dimension(); }
@@ -82,8 +84,8 @@ class nn : public component, public data_receiver, public data_provider, public
  virtual string description ();
  string outline (bool show_first_index_as_one=false);                   // output a textual summary of the NN structure
  string item_description (int item);
- void from_stream ( std::istream REF s );			        // overides virtual method in component, only reads header
- void to_stream   ( std::ostream REF s );			        // overides virtual method in component
+ void from_stream ( std::istream REF s );			        // overrides virtual method in component, only reads header
+ void to_stream   ( std::ostream REF s );			        // overrides virtual method in component
 
  bool set_component_for_input(int index);                               // set which component in the topology is used for input (by index position in topology)
  bool set_component_for_input_by_id(int id);                            // set which component in the topology is used for input (by component id)
@@ -99,6 +101,7 @@ class nn : public component, public data_receiver, public data_provider, public
  bool add_component      (component      * p_component);                // append any component to topology
  bool add_layer          (layer          * p_layer);                    // append a layer to topology
  bool add_connection_set (connection_set * p_connection_set);           // append a connection_set to topology
+ bool add_aux_control    (aux_control    * p_aux_control);
 
  bool connect_consecutive_layers(bool fully_connect = true,             // locate layer+connection_set+layer sequences in topology and create internal connections
                                  bool set_ready_to_encode_fwd = true,
@@ -116,7 +119,7 @@ class nn : public component, public data_receiver, public data_provider, public
  bool connect_layers_at_topology_indexes(                               // connect two layers using their current position in topology:
                                   int source_layer_index,               // current index position (in topology) of source layer
                                   int destination_layer_index,          // current index position (in topology) of destination layer
-                                  connection_set PTR p_connection_set,  // preallocated new connection set to use for connecting the layers (will be inserted in topology after source layer)
+                                  connection_set PTR p_connection_set,  // pre-allocated new connection set to use for connecting the layers (will be inserted in topology after source layer)
                                   bool fully_connect,                   // create connections between all pes in layers
                                   DATA min_random_weight=0,             // if creating connections, randomize weights between...
                                   DATA max_random_weight=0);            // these two values.
@@ -126,18 +129,25 @@ class nn : public component, public data_receiver, public data_provider, public
 
  component * component_from_id(int id);                                 // returns a pointer to component with given id in topology, NULL if not available
  component * component_from_topology_index(int index);                  // returns a pointer to the "index"-th component in topology, NULL if not available
+ bool component_accepts_input(int index);						    	// returns true if data_receiver (layer/aux_control)
+ bool component_provides_output(int index);					        	// returns true if data_provider (layer/aux_control)
 
  // some patches that may be useful for interactive use (s.a. in nnlib2Rcpp R package):
 
  layer PTR get_layer_at(int index);                                     // NULL if not found or not layer
  connection_set PTR get_connection_set_at(int index);                   // NULL if not found or not connection_set
+ aux_control PTR get_aux_control_at(int index);                         // NULL if not found or not aux_control
  bool add_connection(int index, int source_pe, int destin_pe, DATA weight);
  bool remove_connection(int index, int connection_number);
- bool get_input_data_at_component (int index, DATA * buffer, int dimension);
+ bool get_input_at_component (int index, DATA * buffer, int dimension);
+ bool set_input_at_component (int index, DATA * data, int dimension);
  bool get_weights_at_component (int index, DATA * buffer, int dimension);
+ bool set_weights_at_component(int index, DATA * data, int dimension);
  DATA get_weight_at_component(int index, int connection_number);        // returns 0 if not successful
  bool set_weight_at_component(int index, int connection_number, DATA weight);
+ bool get_misc_at_component(int index, DATA * buffer, int dimension);
  bool set_misc_at_component(int index, DATA * data, int dimension);
+ bool get_output_from_component (int index, DATA * buffer, int dimension);
  bool set_output_at_component(int index, DATA * data, int dimension);   // currently only works only for layers
  bool get_biases_at_component(int index, DATA * buffer, int dimension); // only works only for layers
  DATA get_bias_at_component(int index, int pe_number);					// only works only for layers
