@@ -35,9 +35,12 @@ class bp_nn : public NN_PARENT_CLASS
  public:
  static bool display_squared_error;		// true = display squared error when encoding, false = display absolute error when encoding
 
+ bool m_use_squared_error;				// as above but per bp_nn instance
+
  DATA bp_rnd_min;
  DATA bp_rnd_max;
 
+
  bp_nn();
  ~bp_nn();
  bool setup(int input_dimension,int output_dimension,DATA learning_rate,int hidden_layers,int hidden_layer_size);
@@ -65,7 +68,7 @@ class bpu1_nn : public bp_nn
  };
 
 /*-----------------------------------------------------------------------*/
-/* Experimental Unsupervised extention of Back Propagation by VNN	 */
+/* Experimental Unsupervised extension of Back Propagation by VNN	 */
 /*                              PunishLoosers				 */
 /*-----------------------------------------------------------------------*/
 
@@ -77,7 +80,7 @@ class bpu2_nn : public bp_nn
  };
 
 /*-----------------------------------------------------------------------*/
-/* Experimental Unsupervised extention of Back Propagation by VNN	 */
+/* Experimental Unsupervised extension of Back Propagation by VNN	 */
 /*                              HardLimitLike			         */
 /*-----------------------------------------------------------------------*/
 
@@ -89,7 +92,7 @@ class bpu3_nn : public bp_nn
  };
 
 /*-----------------------------------------------------------------------*/
-/* Experimental Unsupervised extention of Back Propagation by VNN	 */
+/* Experimental Unsupervised extension of Back Propagation by VNN	 */
 /*	Autoencoder1 (was: VNN's I Think I Saw A Pussy Cat 1)		 */
 /*-----------------------------------------------------------------------*/
 
@@ -112,7 +115,7 @@ class bpu4_nn : public bp_nn
  };
 
 /*-----------------------------------------------------------------------*/
-/* Experimental extention of Back Propagation by VNN			 */
+/* Experimental extension of Back Propagation by VNN			 */
 /* I called it 	Heteroencoder (to use for dimensionality reduction)...	 */
 /*-----------------------------------------------------------------------*/
 /* ...later I realized, such NN are called AutoEncoders in bibliography	 */
