@@ -33,28 +33,29 @@ class pe
 
  public:
 
- DATA input;					        // final input to this pe, may be accessed directly or result from input_function.
- DATA bias;					        // bias
- DATA output;					        // output of this pe.
- DATA misc;						// helper register for misc use.
+ DATA input;					        				// final input to this pe, may be accessed directly or result from input_function.
+ DATA bias;					        					// bias
+ DATA output;					        				// output of this pe.
+ DATA misc;												// helper register for misc use.
 
  pe();
  ~pe();
 
- void reset();						// partial reset of this pe.
+ void reset();											// partial reset of this pe.
  string description();                                  // text containing status and information of PE
 
  virtual DATA input_function();                         // optional: overload with input function, default: sums received_values and puts result to input variable
  virtual DATA activation_function(DATA value);          // optional: overload with activation function, otherwise returns value
  virtual DATA threshold_function(DATA value);           // optional: overload with threshold function, otherwise returns value
 
- DATA add_to_input (DATA value);        	        // directly add (arithmeticaly) the value to input (summation function)
+ DATA add_to_input (DATA value);        	    		// directly add (arithmeticaly) the value to input (summation function)
  bool receive_input_value (DATA value);                 // append to list of input values (received_values), later to be processed by input_function for producing final input value.
  int  reset_received_values();                          // empties list of received input values (received_values).
  void move_input_to_output();                           // sometimes useful, also sets input to 0 and resets received values.
+ DATA preview_current_input();							// The pe's preview_current_input() method is a PATCH to _estimate_ current input value without changing the pe's current state.
 
- virtual void encode();                                 // should be overiden by derived classes. Default employs input->activation->threshold function sequence and places results to output
- virtual void recall();                                 // should be overiden by derived classes. Default employs input->activation->threshold function sequence and places results to output
+ virtual void encode();                                 // should be overridden by derived classes. Default employs input->activation->threshold function sequence and places results to output
+ virtual void recall();                                 // should be overridden by derived classes. Default employs input->activation->threshold function sequence and places results to output
 
  friend std::istream& operator>> ( std::istream REF s, pe REF it );
  friend std::ostream& operator<< ( std::ostream REF s, pe REF it );
